@@ -39,6 +39,7 @@ this program; if not, write to the Free Software Foundation, Inc.,
 #include "sync0sync.h"
 #include "univ.i"
 #include "ut0counter.h"
+#include "ut0log.h"
 #include "ut0new.h"
 
 #ifdef UNIV_DEBUG
@@ -54,24 +55,6 @@ typedef CRITICAL_SECTION sys_mutex_t;
 typedef pthread_mutex_t sys_mutex_t;
 #endif /* _WIN32 */
 
-/** The new (C++11) syntax allows the following and we should use it when it
-is available on platforms that we support.
-
-        enum class mutex_state_t : lock_word_t { ... };
-*/
-
-/** Mutex states. */
-enum mutex_state_t {
-  /** Mutex is free */
-  MUTEX_STATE_UNLOCKED = 0,
-
-  /** Mutex is acquired by some thread. */
-  MUTEX_STATE_LOCKED = 1,
-
-  /** Mutex is contended and there are threads waiting on the lock. */
-  MUTEX_STATE_WAITERS = 2
-};
-
 /*
                 LATCHING ORDER WITHIN THE DATABASE
                 ==================================
@@ -271,12 +254,14 @@ enum latch_level_t {
   SYNC_LOG_ARCH,
 
   SYNC_PAGE_CLEANER,
-  SYNC_PURGE_QUEUE,
   SYNC_TRX_SYS_HEADER,
+  SYNC_TRX_SYS_SERIALISATION,
+  SYNC_PURGE_QUEUE,
   SYNC_THREADS,
   SYNC_TRX,
   SYNC_POOL,
   SYNC_POOL_MANAGER,
+  SYNC_TRX_SYS_SHARD,
   SYNC_TRX_SYS,
   SYNC_LOCK_SYS_SHARDED,
   SYNC_LOCK_SYS_GLOBAL,
@@ -363,6 +348,7 @@ enum latch_id_t {
   LATCH_ID_BUF_POOL_FLUSH_STATE,
   LATCH_ID_DBLWR,
   LATCH_ID_DBLWR_SPACE_CACHE,
+  LATCH_ID_DDL_AUTOINC,
   LATCH_ID_CACHE_LAST_READ,
   LATCH_ID_DICT_FOREIGN_ERR,
   LATCH_ID_DICT_SYS,
@@ -425,6 +411,8 @@ enum latch_id_t {
   LATCH_ID_TEMP_POOL_MANAGER,
   LATCH_ID_TRX,
   LATCH_ID_TRX_SYS,
+  LATCH_ID_TRX_SYS_SHARD,
+  LATCH_ID_TRX_SYS_SERIALISATION,
   LATCH_ID_SRV_SYS,
   LATCH_ID_SRV_SYS_TASKS,
   LATCH_ID_PAGE_ZIP_STAT_PER_INDEX,
@@ -497,7 +485,7 @@ struct OSMutex {
   }
 
   /** Destructor */
-  ~OSMutex() {}
+  ~OSMutex() = default;
 
   /** Destroy the mutex */
   void destroy() UNIV_NOTHROW {
@@ -583,9 +571,9 @@ Used for mutexes which have PFS keys defined under UNIV_PFS_MUTEX.
 @param[in]	id		Latch id
 @param[in]	level		Latch level
 @param[in]	key		PFS key */
-#define LATCH_ADD_MUTEX(id, level, key) \
-  latch_meta[LATCH_ID_##id] =           \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level, key))
+#define LATCH_ADD_MUTEX(id, level, key)                      \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level, key)
 
 #ifdef UNIV_PFS_RWLOCK
 /** Latch element.
@@ -593,22 +581,23 @@ Used for rwlocks which have PFS keys defined under UNIV_PFS_RWLOCK.
 @param[in]	id		Latch id
 @param[in]	level		Latch level
 @param[in]	key		PFS key */
-#define LATCH_ADD_RWLOCK(id, level, key) \
-  latch_meta[LATCH_ID_##id] =            \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level, key))
+#define LATCH_ADD_RWLOCK(id, level, key)                     \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level, key)
 #else
-#define LATCH_ADD_RWLOCK(id, level, key)    \
-  latch_meta[LATCH_ID_##id] = UT_NEW_NOKEY( \
-      latch_meta_t(LATCH_ID_##id, #id, level, #level, PSI_NOT_INSTRUMENTED))
+#define LATCH_ADD_RWLOCK(id, level, key)                                     \
+  latch_meta[LATCH_ID_##id] =                                                \
+      ut::new_withkey<latch_meta_t>(UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, \
+                                    #id, level, #level, PSI_NOT_INSTRUMENTED)
 #endif /* UNIV_PFS_RWLOCK */
 
 #else
-#define LATCH_ADD_MUTEX(id, level, key) \
-  latch_meta[LATCH_ID_##id] =           \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level))
-#define LATCH_ADD_RWLOCK(id, level, key) \
-  latch_meta[LATCH_ID_##id] =            \
-      UT_NEW_NOKEY(latch_meta_t(LATCH_ID_##id, #id, level, #level))
+#define LATCH_ADD_MUTEX(id, level, key)                      \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level)
+#define LATCH_ADD_RWLOCK(id, level, key)                     \
+  latch_meta[LATCH_ID_##id] = ut::new_withkey<latch_meta_t>( \
+      UT_NEW_THIS_FILE_PSI_KEY, LATCH_ID_##id, #id, level, #level)
 #endif /* UNIV_PFS_MUTEX */
 
 /** Default latch counter */
@@ -649,7 +638,7 @@ class LatchCounter {
     m_mutex.destroy();
 
     for (Count *count : m_counters) {
-      UT_DELETE(count);
+      ut::delete_(count);
     }
   }
 
@@ -674,7 +663,7 @@ class LatchCounter {
     Count *count;
 
     if (m_counters.empty()) {
-      count = UT_NEW_NOKEY(Count());
+      count = ut::new_withkey<Count>(UT_NEW_THIS_FILE_PSI_KEY);
       m_counters.push_back(count);
     } else {
       ut_a(m_counters.size() == 1);
@@ -793,7 +782,7 @@ class LatchMeta {
   }
 
   /** Destructor */
-  ~LatchMeta() {}
+  ~LatchMeta() = default;
 
   /** Constructor
   @param[in]	id		Latch id
@@ -870,7 +859,8 @@ class LatchMeta {
 };
 
 typedef LatchMeta<LatchCounter> latch_meta_t;
-typedef std::vector<latch_meta_t *, ut_allocator<latch_meta_t *>> LatchMetaData;
+typedef std::vector<latch_meta_t *, ut::allocator<latch_meta_t *>>
+    LatchMetaData;
 
 /** Note: This is accessed without any mutex protection. It is initialised
 at startup and elements should not be added to or removed from it after
@@ -974,7 +964,7 @@ struct latch_t {
   latch_t &operator=(const latch_t &) = default;
 
   /** Destructor */
-  virtual ~latch_t() UNIV_NOTHROW {}
+  virtual ~latch_t() UNIV_NOTHROW = default;
 
   /** @return the latch ID */
   latch_id_t get_id() const { return (m_id); }
@@ -1046,7 +1036,7 @@ struct latch_t {
 
 /** Subclass this to iterate over a thread's acquired latch levels. */
 struct sync_check_functor_t {
-  virtual ~sync_check_functor_t() {}
+  virtual ~sync_check_functor_t() = default;
   virtual bool operator()(const latch_level_t) = 0;
   virtual bool result() const = 0;
 };
@@ -1059,7 +1049,7 @@ struct btrsea_sync_check : public sync_check_functor_t {
       : m_result(), m_has_search_latch(has_search_latch) {}
 
   /** Destructor */
-  ~btrsea_sync_check() override {}
+  ~btrsea_sync_check() override = default;
 
   /** Called for every latch owned by the calling thread.
   @param[in]	level		Level of the existing latch
@@ -1122,7 +1112,7 @@ struct dict_sync_check : public sync_check_functor_t {
       : m_result(), m_dict_mutex_allowed(dict_mutex_allowed) {}
 
   /** Destructor */
-  ~dict_sync_check() override {}
+  ~dict_sync_check() override = default;
 
   /** Check the latching constraints
   @param[in]	level		The level held by the thread */
@@ -1166,6 +1156,9 @@ struct sync_allowed_latches : public sync_check_functor_t {
   sync_allowed_latches(const latch_level_t *from, const latch_level_t *to)
       : m_result(), m_latches(from, to) {}
 
+  /** Default constructor.  The list of allowed latches is empty. */
+  sync_allowed_latches() : m_result(), m_latches() {}
+
   /** Check whether the given latch_t violates the latch constraint.
   This object maintains a list of allowed latch levels, and if the given
   latch belongs to a latch level that is not there in the allowed list,
@@ -1201,7 +1194,7 @@ struct sync_allowed_latches : public sync_check_functor_t {
   True if all OK */
   bool m_result;
 
-  typedef std::vector<latch_level_t, ut_allocator<latch_level_t>> latches_t;
+  typedef std::vector<latch_level_t, ut::allocator<latch_level_t>> latches_t;
 
   /** List of latch levels that are allowed to be held */
   latches_t m_latches;
